"""Url router for the REST API
"""

from django.urls import re_path

from core_curate_app.rest.curate_data_structure.views import (
    AdminCurateDataStructureList,
    CurateDataStructureList,
    CurateDataStructureDetail,
)

urlpatterns = [
    re_path(
        r"^admin/draft/$",
        AdminCurateDataStructureList.as_view(),
        name="core_curate_app_rest_admin_drafts",
    ),
    re_path(
        r"^draft/$",
        CurateDataStructureList.as_view(),
        name="core_curate_app_rest_drafts",
    ),
    re_path(
        r"^draft/(?P<pk>\w+)/$",
        CurateDataStructureDetail.as_view(),
        name="core_curate_app_rest_draft_detail",
    ),
]



""" Integration Test for Data Structure Rest API
"""

from tests.components.curate_data_structure.fixtures.fixtures import (
    DataStructureFixtures,
)
from core_main_app.utils.tests_tools.MockUser import create_mock_user
from core_main_app.utils.tests_tools.RequestMock import RequestMock
from core_curate_app.rest.curate_data_structure import (
    views as data_structure_rest_views,
)
from rest_framework import status

from core_main_app.utils.integration_tests.integration_base_test_case import (
    MongoIntegrationBaseTestCase,
)

fixture_data_structure = DataStructureFixtures()


class TestDataStructureListAdmin(MongoIntegrationBaseTestCase):
    fixture = fixture_data_structure

    def setUp(self):
        super(TestDataStructureListAdmin, self).setUp()

        self.user = create_mock_user("1", is_staff=True, is_superuser=True)

        self.data = {
            "user": "1",
            "name": "name",
            "form_string": "<tag></tag>",
            "template": str(self.fixture.template.id),
        }

    def test_get_returns_all_user_data_structure(self):
        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(), self.user
        )

        # Assert
        self.assertEqual(len(response.data), 3)

    def test_post_returns_data_structure(self):
        # Arrange

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.data["user"], self.user.id)

        self.assertEqual(response.data["template"], str(self.fixture.template.id))

        self.assertEqual(response.data["form_string"], "<tag></tag>")

    def test_post_incorrect_parameter_returns_http_400(self):
        # Arrange
        self.data["template"] = "507f1f77bcf86cd799439011"

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_data_structure_missing_field_returns_http_400(self):
        # Arrange
        self.data = {"user": "1", "name": "name", "form_string": "<tag></tag>"}

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.AdminCurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataStructureList(MongoIntegrationBaseTestCase):
    fixture = fixture_data_structure

    def setUp(self):
        super(TestDataStructureList, self).setUp()

        self.superuser = create_mock_user("1", is_staff=True, is_superuser=True)

        self.user = create_mock_user("2")

        self.data = {
            "user": "2",
            "name": "name",
            "form_string": "<tag></tag>",
            "template": str(self.fixture.template.id),
        }

    def test_get_returns_all_user_data_structure_as_superuser(self):
        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureList.as_view(), self.superuser
        )

        # Assert
        self.assertEqual(len(response.data), 2)

    def test_get_returns_all_user_data_structure_as_user(self):
        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureList.as_view(), self.user
        )

        # Assert
        self.assertEqual(len(response.data), 1)

    def test_post_returns_http_201(self):
        # Arrange

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.CurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

    def test_post_incorrect_parameter_returns_http_400(self):
        # Arrange
        self.data["template"] = "507f1f77bcf86cd799439011"

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.CurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_post_data_structure_missing_field_returns_http_400(self):
        # Arrange
        self.data = {"user": "1", "name": "name", "form_string": "<tag></tag>"}

        # Act
        response = RequestMock.do_request_post(
            data_structure_rest_views.CurateDataStructureList.as_view(),
            self.user,
            data=self.data,
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)


class TestDataStructureDetail(MongoIntegrationBaseTestCase):
    fixture = fixture_data_structure

    def setUp(self):
        super(TestDataStructureDetail, self).setUp()

    def test_get_returns_http_200(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_get_returns_data_structure(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.data["name"], self.fixture.data_structure_1.name)

    def test_get_other_user_data_structure_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_get_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_get(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_returns_http_204(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_delete_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_delete_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_delete(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_delete_other_user_blob_returns_http_403(self):
        # Arrange
        user = create_mock_user("2")

        # Act
        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_patch_returns_updated_name(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": str(self.fixture.data_structure_1.id)},
            data={"name": "new_name"},
        )

        # Assert
        self.assertEqual(response.data["name"], "new_name")

    def test_patch_wrong_id_returns_http_404(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            param={"pk": "507f1f77bcf86cd799439011"},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_patch_wrong_template_returns_http_400(self):
        # Arrange
        user = create_mock_user("1")

        # Act
        response = RequestMock.do_request_patch(
            data_structure_rest_views.CurateDataStructureDetail.as_view(),
            user,
            data={"template": "507f1f77bcf86cd799439011"},
            param={"pk": self.fixture.data_structure_1.id},
        )

        # Assert
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)



"""Core Curate App Settings

Settings with the following syntax can be overwritten at the project level:
SETTING_NAME = getattr(settings, "SETTING_NAME", "Default Value")
"""
import os

from django.conf import settings

if not settings.configured:
    settings.configure()

INSTALLED_APPS = getattr(settings, "INSTALLED_APPS", [])

PARSER_MIN_TREE = getattr(settings, "PARSER_MIN_TREE", True)
PARSER_IGNORE_MODULES = getattr(settings, "PARSER_IGNORE_MODULES", False)
PARSER_COLLAPSE = getattr(settings, "PARSER_COLLAPSE", True)
PARSER_AUTO_KEY_KEYREF = getattr(settings, "PARSER_AUTO_KEY_KEYREF", False)
PARSER_IMPLICIT_EXTENSION_BASE = getattr(
    settings, "PARSER_IMPLICIT_EXTENSION_BASE", False
)
PARSER_DOWNLOAD_DEPENDENCIES = getattr(settings, "PARSER_DOWNLOAD_DEPENDENCIES", False)

# MENU
CURATE_MENU_NAME = getattr(settings, "CURATE_MENU_NAME", "Curator")

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
LOCALE_PATHS = (os.path.join(BASE_DIR, "core_curate_app/locale"),)

ENABLE_XML_ENTITIES_TOOLTIPS = getattr(settings, "ENABLE_XML_ENTITIES_TOOLTIPS", True)
""" boolean: enable the xml entities warning tooltip on the GUI.
"""



"""Curate Data Structure models
"""

from django_mongoengine import fields
from mongoengine import errors as mongoengine_errors
from mongoengine.errors import NotUniqueError
from mongoengine.queryset.base import CASCADE

from core_curate_app.permissions import rights
from core_main_app.commons import exceptions
from core_main_app.components.data.models import Data
from core_parser_app.components.data_structure.models import DataStructure
from signals_utils.signals.mongo import connector, signals


class CurateDataStructure(DataStructure):
    """Curate data structure."""

    form_string = fields.StringField(blank=True)
    data = fields.ReferenceField(Data, blank=True, reverse_delete_rule=CASCADE)

    @staticmethod
    def get_permission():
        return f"{rights.curate_content_type}.{rights.curate_data_structure_access}"

    def save_object(self):
        """Custom save

        Returns:

        """
        try:
            return self.save()
        except NotUniqueError:
            raise exceptions.ModelError("Unable to save the document: not unique.")
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_by_id(data_structure_id):
        """Return the object with the given id.

        Args:
            data_structure_id:

        Returns:
            Curate Data Structure (obj): CurateDataStructure object with the given id

        """
        try:
            return CurateDataStructure.objects.get(pk=str(data_structure_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_none():
        """Return None object, used by forms.

        Returns:

        """
        return CurateDataStructure.objects().none()

    @staticmethod
    def get_all():
        """Return all curate data structure api.

        Returns:

        """
        return CurateDataStructure.objects.all()

    @staticmethod
    def get_all_by_user_id_and_template_id(user_id, template_id):
        """Return all template version managers with user set to None.

        Returns:

        """
        return CurateDataStructure.objects(
            user=str(user_id), template=str(template_id)
        ).all()

    @staticmethod
    def get_by_user_id_and_template_id_and_name(user_id, template_id, name):
        """Return the curate data structure with user, template id and name.

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(
                user=str(user_id), template=str(template_id), name=name
            )
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_all_by_user_id_with_no_data(user_id):
        """
        Return all the curate date structure of the user, with no data.

        Args: user_id:
        Return:
        """
        return CurateDataStructure.objects(user=str(user_id), data__exists=False).all()

    @staticmethod
    def get_all_except_user_id_with_no_data(user_id):
        """Return all the curate date structure except the one of the user, with no data.

        Args: user_id:
        Return:
        """
        return CurateDataStructure.objects(
            user__ne=str(user_id), data__exists=False
        ).all()

    @staticmethod
    def get_all_by_user_id_and_template_id_with_no_data(user_id, template_id):
        """

        Args:
            user_id:
            template_id:

        Returns:

        """
        return CurateDataStructure.objects(
            user=str(user_id), template=str(template_id), data__exists=False
        ).all()

    @staticmethod
    def get_all_with_no_data():
        """Returns all curate data structure api with no link to a data.

        Args:

        Returns:

        """
        return CurateDataStructure.objects(data__exists=False).all()

    @staticmethod
    def get_all_by_user(user_id):
        """Return all curate data structure by user.

        Returns:

        """
        return CurateDataStructure.objects(user=str(user_id)).all()

    @staticmethod
    def get_by_data_id(data_id):
        """Return the curate data structure with the given data id

        Args:
            data_id:

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(data=str(data_id))
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))

    @staticmethod
    def get_by_data_structure_element_root(data_structure_element_root):
        """Return the curate data structure with the given data id

        Args:
            data_structure_element_root:

        Returns:

        """
        try:
            return CurateDataStructure.objects.get(
                data_structure_element_root=str(data_structure_element_root.id)
            )
        except mongoengine_errors.DoesNotExist as e:
            raise exceptions.DoesNotExist(str(e))
        except Exception as ex:
            raise exceptions.ModelError(str(ex))


# Connect signals
connector.connect(DataStructure.pre_delete, signals.pre_delete, CurateDataStructure)



"""Curate Data Structure Serializers
"""

from rest_framework_mongoengine.serializers import DocumentSerializer

from core_curate_app.components.curate_data_structure import (
    api as curate_data_structure_api,
)
from core_curate_app.components.curate_data_structure.models import CurateDataStructure


class CurateDataStructureSerializer(DocumentSerializer):
    """CurateDataStructure Serializer"""

    class Meta(object):
        """Meta"""

        model = CurateDataStructure
        fields = "__all__"

    def create(self, validated_data):
        """
        Create and return a new `CurateDataStructure` instance, given the validated data.
        """
        # Create data
        curate_data_structure = CurateDataStructure(
            name=validated_data["name"],
            template=validated_data["template"],
            form_string=validated_data["form_string"]
            if "form_string" in validated_data
            else None,
            user=str(self.context["request"].user.id),
            data=validated_data["data"] if "data" in validated_data else None,
            data_structure_element_root=validated_data["data_structure_element_root"]
            if "data_structure_element_root" in validated_data
            else None,
        )

        return curate_data_structure_api.upsert(
            curate_data_structure, self.context["request"].user
        )

    def update(self, instance, validated_data):
        """
        Update and return an existing `CurateDataStructure` instance, given the validated data.
        """
        instance.name = validated_data.get("name", instance.name)
        instance.form_string = validated_data.get("form_string", instance.form_string)
        instance.data_structure_element_root = validated_data.get(
            "data_structure_element_root", instance.data_structure_element_root
        )
        instance.data = validated_data.get("data", instance.data)

        return curate_data_structure_api.upsert(instance, self.context["request"].user)



"""Curate Data Structure Serializers
"""

from rest_framework_mongoengine.serializers import DocumentSerializer

from core_curate_app.components.curate_data_structure import (
    api as curate_data_structure_api,
)
from core_curate_app.components.curate_data_structure.models import CurateDataStructure


class CurateDataStructureAdminSerializer(DocumentSerializer):
    """CurateDataStructure Serializer"""

    class Meta(object):
        """Meta"""

        model = CurateDataStructure
        fields = "__all__"

    def create(self, validated_data):
        """
        Create and return a new `CurateDataStructure` instance, given the validated data.
        """
        # Create data
        curate_data_structure = CurateDataStructure(
            user=validated_data["user"],
            name=validated_data["name"],
            template=validated_data["template"],
            form_string=validated_data["form_string"]
            if "form_string" in validated_data
            else None,
            data=validated_data["data"] if "data" in validated_data else None,
            data_structure_element_root=validated_data["data_structure_element_root"]
            if "data_structure_element_root" in validated_data
            else None,
        )
        return curate_data_structure_api.upsert(
            curate_data_structure, self.context["request"].user
        )



"""Curate models
"""

from django.db import models

from core_curate_app.permissions import rights
from core_main_app.permissions.utils import get_formatted_name


class Curate(models.Model):
    class Meta(object):
        verbose_name = "core_curate_app"
        default_permissions = ()
        permissions = (
            (rights.curate_access, get_formatted_name(rights.curate_access)),
            (
                rights.curate_view_data_save_repo,
                get_formatted_name(rights.curate_view_data_save_repo),
            ),
            (
                rights.curate_data_structure_access,
                get_formatted_name(rights.curate_data_structure_access),
            ),
        )



"""
    Common views
"""
from django.utils.translation import ugettext as _

from core_curate_app.views.user.views import _get_curate_data_structure_by_id
from core_main_app.views.common.views import CommonView


class FormView(CommonView):
    """
    Form View
    """

    template = "core_curate_app/user/detail.html"

    def get(self, request, *args, **kwargs):
        try:
            # get data structure
            curate_data_structure = _get_curate_data_structure_by_id(
                kwargs["curate_data_structure_id"], request
            )

            # TODO: fix with the rework on the curate workflow
            if curate_data_structure.form_string is None:
                raise Exception(
                    "The "
                    + _("form_label")
                    + " was not saved. We can't display the correct data."
                )

            # Set the assets
            assets = {
                "js": [
                    {"path": "core_main_app/common/js/XMLTree.js", "is_raw": False},
                    {"path": "core_main_app/user/js/data/detail.js", "is_raw": False},
                ],
                "css": ["core_main_app/common/css/XMLTree.css"],
            }

            # Set the context
            context = {
                "data_structure": curate_data_structure,
            }

            return self.common_render(
                request, self.template, assets=assets, context=context
            )
        except Exception as e:
            template = "core_main_app/common/commons/error.html"
            if self.administration:
                template = "core_main_app/admin/commons/errors/errors_wrapper.html"

            return self.common_render(request, template, context={"error": str(e)})



""" Rights for core curate app
"""

curate_content_type = "core_curate_app"
curate_access = "access_curate"
curate_view_data_save_repo = "view_data_save_repo"
curate_data_structure_access = "access_curate_data_structure"
